package com.scbfkj.uni.library.script;

import com.fasterxml.jackson.databind.JsonNode;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.service.LoggerService;
import org.apache.commons.pool2.BaseKeyedPooledObjectFactory;
import org.apache.commons.pool2.KeyedObjectPool;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.commons.pool2.impl.GenericKeyedObjectPool;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

public class JavaScriptEngineUtil {

    private static class JavaApply {
        private Object instance;
        private Method apply;
        private Method close;

        public JavaApply(Object instance, Method apply, Method close) {
            this.instance = instance;
            this.apply = apply;
            this.close = close;
        }

        public Object getInstance() {
            return instance;
        }

        public void setInstance(Object instance) {
            this.instance = instance;
        }

        public Method getApply() {
            return apply;
        }

        public void setApply(Method apply) {
            this.apply = apply;
        }

        public Method getClose() {
            return close;
        }

        public void setClose(Method close) {
            this.close = close;
        }

        public void invokeClose() throws InvocationTargetException, IllegalAccessException {
            if (Objects.nonNull(close)) {
                close.invoke(instance);
            }
        }

        public Object invokeApply(Object... args) throws InvocationTargetException, IllegalAccessException {
            return apply.invoke(instance, args);
        }
    }

    private final static KeyedObjectPool<String, JavaApply> javaRefPool = new GenericKeyedObjectPool<>(new BaseKeyedPooledObjectFactory<String, JavaApply>() {
        @Override
        public JavaApply create(String key) throws Exception {
            JsonNode jsonNode = DataFormatUtil.toJsonNode(key);
            JsonNode path = jsonNode.get("path");
            JsonNode className = jsonNode.get("className");
            JsonNode method = jsonNode.get("methodName");
            ClassLoader classLoader=null;
            try{
                if (Objects.nonNull(path) && !path.isNull()) {
                    String userPath = System.getProperty("user.dir");
                    File file = new File(userPath.concat(File.separator).concat("plugins").concat(File.separator).concat(path.asText()));
                    if (!file.exists()) {
                        throw new RuntimeException("外部文件加载不存在：".concat(userPath).concat(File.separator).concat("plugins").concat(File.separator).concat(path.asText()));
                    }
                    URL url = file.toURI().toURL();
                    classLoader = new URLClassLoader(new URL[]{url},Thread.currentThread().getContextClassLoader());
                } else {
                    classLoader = Thread.currentThread().getContextClassLoader();
                }

                Class<?> classExample = classLoader.loadClass(className.asText()); //获取类实例
                Object classInstance = classExample.getConstructor().newInstance();//类实例接口 无参数构造
                Method javaMethod = null;
                Method closeMethod = null;
                for (Method currentMethod : classExample.getMethods()) {//循环所有方法
                    String methodName = currentMethod.getName();
                    if (methodName.equals(method.asText())) {
                        javaMethod = currentMethod;
                    } else if (methodName.equals("close")) {
                        closeMethod = currentMethod;
                    }
                }
                return new JavaApply(classInstance, javaMethod, closeMethod);
            }finally {
                if(Objects.nonNull(classLoader)){
                    if (classLoader instanceof URLClassLoader cl) {
                        cl.close();
                    }
                }
            }
        }

        @Override
        public PooledObject<JavaApply> wrap(JavaApply value) {
            return new DefaultPooledObject<>(value);
        }

        @Override
        public void destroyObject(String key, PooledObject<JavaApply> p) throws Exception {
            JavaApply javaApply = p.getObject();
            javaApply.invokeClose();

        }
    });

    public static Map<String, Object> invoke(Map<String, Object> datasource, String expression, Object... args) throws Exception {
        datasource.put("methodName", expression);
        String key = DataFormatUtil.toString(datasource);

        JavaApply javaApply = null;
        try {
            javaApply = javaRefPool.borrowObject(key);
            return UniReturnUtil.success(javaApply.invokeApply(args));
        } catch (Exception e) {
            e.printStackTrace();
            return UniReturnUtil.fail(e);
        } finally {
            if (Objects.nonNull(javaApply)) {
                javaRefPool.returnObject(key, javaApply);
            }
        }
    }
}
